library convert_mate_code;

import 'dart:io';

import 'package:path/path.dart' as Path;

import 'mate/font.dart';
import 'mate/icon.dart';
import 'mate/image.dart';
import 'mate/string.dart';
import 'mate/svg.dart';
import '../../../logger/log.dart';

var imageType = <String>{'.jpg', '.png', '.jpeg'};
var svgType = <String>{'.svg'};
var stringType = <String>{'.yaml', '.json', '.txt'}; // 字符串类型，都随 _R_Text 导入项目，yaml，json等额外特殊处理到String类型

var fontType = <String>{'.ttf', '.otf'};
var audioType = <String>{'.mp3', '.opus', '.wav', '.pcm'};
var videoType = <String>{'.mp4', '.avi'};
var binaryType = <String>{'.bin', '.so', ''};

class CodeAndPubSpecYaml {
  final Set<MateImage> mateImages;
  final Set<MateSvg> mateSvgs;
  final Set<MateString> mateStrings;
  final Set<MateFont> mateFonts;
  final Set<MateIcon> svgToIcons;

  CodeAndPubSpecYaml(this.mateImages, this.mateSvgs, this.mateStrings, this.mateFonts, this.svgToIcons);
}

CodeAndPubSpecYaml geneCodeAndPubSpecYaml(String rootPath) {
  mirrorLog.info("geneCodeAndPubSpecYaml rootPath = ${rootPath}");
  var mateImages = <MateImage>{};
  var mateSvgs = <MateSvg>{};
  var mateStrings = <MateString>{};
  var mateFonts = <MateFont>{};
  var svgToIcons = <MateIcon>{};

  Directory.current = rootPath;
  var dirAssets = Directory('assets/');
  dirAssets.createSync(recursive: true);
  List allAssets = dirAssets.listSync(recursive: true);
  allAssets.forEach((element) {
    if (element is File) {
      mirrorLog.info("geneCodeAndPubSpecYaml filename = ${Path.basename(element.path)}");
      // String filename = Path.basename(element.path);
      // String name = Path.basenameWithoutExtension(element.path);
      String ext = Path.extension(element.path).toLowerCase();
      if (fontType.contains(ext)) {
        var tempFont = convertFont(element);
        if (mateFonts.contains(tempFont)) {
          var oldFont = mateImages.lookup(tempFont);
          throw FormatException("""文件名转换变量名冲突，${tempFont.generatedVarName}，
              不能导入 ${Uri.base}${tempFont.assetFilePath}, 
              文件 ${Uri.base}${oldFont.assetFilePath} 优先级更高""");
        } else {
          mateFonts.add(tempFont);
        }
        return;
      }
      if (imageType.contains(ext)) {
        var tempImg = convertImage(element);
        if (mateImages.contains(tempImg)) {
          var oldImg = mateImages.lookup(tempImg);
          throw FormatException("""文件名转换变量名冲突，${tempImg.generatedVarName}，
              不能导入 ${Uri.base}${tempImg.assetFilePath}, 
              文件 ${Uri.base}${oldImg.assetFilePath} 优先级更高""");
        } else {
          mateImages.add(tempImg);
        }
        return;
      }
      if (svgType.contains(ext)) {
        var tempSvg = convertSvg(element);
        if (mateSvgs.contains(tempSvg)) {
          var oldSvg = mateSvgs.lookup(tempSvg);
          throw FormatException("""文件名转换变量名冲突，${tempSvg.generatedVarName}，
              不能导入 ${Uri.base}${tempSvg.assetFilePath}, 
              文件 ${Uri.base}${oldSvg.assetFilePath} 优先级更高""");
        } else {
          mateSvgs.add(tempSvg);
        }
        // 是svg，刚好父路径是icons，转换为可直接使用的icon
        var fakePath = element.path.replaceAll(Path.basename(element.path), '');
        if (fakePath == 'assets/icons/') {
          var tempIcon = convertIcon(element);
          if (svgToIcons.contains(tempIcon)) {
            var oldIcon = svgToIcons.lookup(tempIcon);
            throw FormatException("""Icons:
              文件名转换变量名冲突，${tempIcon.generatedVarName}，
              不能导入 ${Uri.base}${tempIcon.assetFilePath}, 
              文件 ${Uri.base}${oldIcon.assetFilePath} 优先级更高""");
          } else {
            svgToIcons.add(tempIcon);
          }
        }
        return;
      }
      if (audioType.contains(ext)) {
        return;
      }
      if (videoType.contains(ext)) {
        return;
      }
      if (stringType.contains(ext)) {
        var tempString = convertString(element);
        if (mateStrings.contains(tempString)) {
          var oldString = mateStrings.lookup(tempString);
          throw FormatException("""文件名转换变量名冲突，${tempString.generatedVarName}，
              不能导入 ${Uri.base}${tempString.assetFilePath}, 
              文件 ${Uri.base}${oldString.assetFilePath} 优先级更高""");
        } else {
          mateStrings.add(tempString);
        }
        return;
      }
      if (binaryType.contains(ext)) {
        return;
      }
    }
  });

  // mateImages.forEach((element) {
  //   print('insertPubSpecYaml ${element}');
  // });

  mirrorLog.info("geneCodeAndPubSpecYaml done");
  return CodeAndPubSpecYaml(mateImages, mateSvgs, mateStrings, mateFonts, svgToIcons);
}
