import 'dart:convert';
import 'dart:io';

import 'package:path/path.dart' as Path;
import 'package:yaml/yaml.dart';

import '../template_string.dart';

const _tCodeTextAssetResource = """  /// asset: {#filePath#}
  // ignore: non_constant_identifier_names
  final {#varName#} = const AssetResource("{#filePath#}", packageName: R.package);
""";

const _tCodeText = """  /// asset: {#filePath#}
  // ignore: non_constant_identifier_names
  Future<String> {#varName#}() {
    final str = rootBundle.loadString(asset.{#varName#}.keyName);
    return str;
  }
  
  String get {#varName#}Sync => _Raw_Text.{#varName#};
""";

const _tCodeRawText = """  /// asset: {#filePath#}
  // ignore: non_constant_identifier_names
  static String get {#varName#} {
    return \"\"\"{#fileData#}\"\"\";
  }
""";

const _tCodeKey4String = """
  String get fileNameWithKey => "valueByKey";
""";
const _tCodeKey4List = """
  List<String> get fileNameWithKeyIfTypeIsList => <String>["value", 'values'];
""";
const _tCodeKey4Map = """
  Map<String, String> get fileNameWithKeyIfTypeIsMap => <String, String>{"value": 'values'};
""";

//<editor-fold desc="转换text\string类型">

/// yaml json 也算txt，并对他们特殊处理为R.string.fileNameWithKey直接使用，
/// 只处理第一级key，value一律转换为String
MateString convertString(File file) {
  var res = MateString();
  res.assetFilePath = file.path;
  final tAsset = TemplateString(_tCodeTextAssetResource);
  final tText = TemplateString(_tCodeText);
  res.generatedVarName = convertVarName(file.path);
  res.generatedCode4Asset = tAsset.format(<String, dynamic>{
    "filePath": file.path,
    'varName': res.generatedVarName,
  });
  res.generatedCode4Text = tText.format(<String, dynamic>{
    "filePath": file.path,
    'varName': res.generatedVarName,
  });

  // 直接读取文件转换文本格式到变量
  var fileData = file.readAsStringSync();
  res.generatedCode4RawText = TemplateString(_tCodeRawText).format(<String, dynamic>{
    "filePath": file.path,
    "fileData": fileData,
    "varName": res.generatedVarName,
  });
  // 额外处理 json yaml
  if (['.json', '.yaml'].contains(Path.extension(file.path))) {
    var tempMap;
    if (['.json'].contains(Path.extension(file.path))) {
      tempMap = json.decode(fileData);
    } else if (['.yaml'].contains(Path.extension(file.path))) {
      tempMap = json.decode(json.encode(loadYaml(fileData)));
    }
    // 根据文件名生成隐藏类代码。_R_String使用变量代理隐藏类
    var tempClass = """class _Hide{#className#}{
      const _Hide{#className#}();
      {#geneCodeByMap#}
    }""";
    var geneCodeByMap = '';
    Map<String, dynamic>.from(tempMap).forEach((key, value) {
      geneCodeByMap += """
      /// ${value}
      String get ${convertVarName(key)} => '${value}';
      """;
    });
    res.generatedCode4MapDataToHideClass = TemplateString(tempClass).format(<String, dynamic>{
      "className": res.generatedVarName,
      "geneCodeByMap": geneCodeByMap,
    });

    res.generatedCode4MapDataToString =
        TemplateString("""_Hide{#className#} get {#className#} => _Hide{#className#}();\n""").format(<String, dynamic>{
      "className": res.generatedVarName,
    });
  }
  return res;
}

class MateString {
  String assetFilePath;
  String generatedVarName;
  String generatedCode4Asset;
  String generatedCode4Text;

  String generatedCode4MapDataToString = '';
  String generatedCode4MapDataToHideClass = '';

  String generatedCode4RawText;

  @override
  String toString() {
    return 'MateImage{assetFilePath: $assetFilePath, generatedVarName: $generatedVarName}';
  }

  @override
  bool operator ==(Object other) {
    return other is MateString && other.generatedVarName == this.generatedVarName;
  }

  @override
  int get hashCode => generatedVarName.hashCode;
}
//</editor-fold>
