library template_string;

import 'dart:convert';

import 'package:basic_utils/basic_utils.dart';
import 'package:path/path.dart' as Path;

class TemplateString {
  final List<String> fixedComponents;
  final Map<int, String> genericComponents;

  int totalComponents;

  TemplateString(String template)
      : fixedComponents = <String>[],
        genericComponents = <int, String>{},
        totalComponents = 0 {
    final List<String> components = template.split('{#');

    for (String component in components) {
      if (component == '') continue; // If the template starts with"{", skip the first element.

      final split = component.split('#}');

      if (split.length != 1) {
        // The condition allows for template strings without parameters.
        genericComponents[totalComponents] = split.first;
        totalComponents++;
      }

      if (split.last != '') {
        fixedComponents.add(split.last);
        totalComponents++;
      }
    }
  }

  String format(Map<String, dynamic> params) {
    String result = '';

    int fixedComponent = 0;
    for (int i = 0; i < totalComponents; i++) {
      if (genericComponents.containsKey(i)) {
        result += '${params[genericComponents[i]]}';
        continue;
      }
      result += fixedComponents[fixedComponent++];
    }

    return result;
  }
}

/// 转换文件名为驼峰命名
///
/// 字符工具包
/// https://pub.dev/packages/basic_utils#cryptoutils
String convertVarName(String filePath) {
  var res;
  var fragments = Path.basename(filePath).split(RegExp(r"[\.\-\_ \&\#+]"));
  var firstWord = fragments[0];
  res = firstWord +
      fragments
          .sublist(1)
          .map((element) {
            if (StringUtils.isAscii(element)) {
              return StringUtils.capitalize(element);
            } else {
              return CryptoUtils.getMd5ThumbprintFromBytes(utf8.encode(element));
            }
          })
          .toList()
          .join();
  return res;
}
