import 'package:analyzer/file_system/file_system.dart';
import 'package:analyzer/src/dart/analysis/driver.dart';
import 'package:analyzer_plugin/plugin/plugin.dart';
import 'package:analyzer_plugin/protocol/protocol_generated.dart';
import 'package:analyzer/src/context/context_root.dart' as analyzer;
import 'package:analyzer/src/context/builder.dart';

import 'logger/log.dart';
import 'package:pub_semver/pub_semver.dart';
import 'generate_r_file/generate_assets_r_code.dart';

/// 盯着dart文件变化，分析是否新增文件，生成对应文件。
class RAssetsPlugin extends ServerPlugin {
  static const excludedFolders = [
    '.dart_tool/**',
  ];

  RAssetsPlugin(ResourceProvider provider) : super(provider) {
    mirrorLog.info("RAssetsPlugin construction");
  }
  @override
  String get name => 'gene R file';

  @override
  String get version => '0.1.0';

  /// 返回一个glob模式列表，选择这个插件感兴趣的文件分析
  @override
  List<String> get fileGlobsToAnalyze => <String>[
        '**/*.png',
        '**/*.jpg',
        '**/*.jpeg',
        '**/*.svg',
        '**/*.json',
        '**/*.yaml',
        // '**/*.dart',
        // 'assets/**/*.svg', // (单独搜assets文件的glob写法没有试出来)
      ];

  /// 这条很重要，false 不兼容，程序就停止了，很多次就是遇到这个坑了
  @override
  bool isCompatibleWith(Version serverVersion) => true;

  /// 修改文件都会触发,新建不触发
  @override
  void contentChanged(String path) {
    mirrorLog.info("contentChanged $path");
    // AnalysisDriverGeneric driver = super.driverForPath(path);
    // driver.addFile(path);
  }

  ContextRoot ctx;

  /// 插件加载时调用
  ///
  /// [contextRoot] 包含排除文件夹、项目所在绝对路径、还有选项文件 analysis_options.yaml 路径
  /// [resourceProvider.pathContext] 提供了访问文件的上下文
  @override
  AnalysisDriverGeneric createAnalysisDriver(ContextRoot contextRoot) {
    ctx = contextRoot;
    generateAssetsDartCode(ctx.root); // 先 生成一次
    mirrorLog.info("createAnalysisDriver");
    // mirrorLog.info("contextRoot ${contextRoot.toJson()}");
    final analysisRoot = analyzer.ContextRoot(
      contextRoot.root,
      // [...contextRoot.exclude, '${contextRoot.root}/generated'], // 生成的文件，不解析
      contextRoot.exclude, // 生成的文件，不解析
      pathContext: resourceProvider.pathContext,
    )..optionsFilePath = contextRoot.optionsFile;

    final contextBuilder = ContextBuilder(resourceProvider, sdkManager, null)
      ..analysisDriverScheduler = analysisDriverScheduler // 主调度器
      ..byteStore = byteStore // 可能是存储cache的意思
      ..performanceLog = performanceLog // 性能日志
      ..fileContentOverlay = fileContentOverlay; // 提供给驱动器的文件内容遮盖（不知overlay啥意思）

    final dartDriver = contextBuilder.buildDriver(analysisRoot);

    return dartDriver;
  }

  @override
  Future<AnalysisHandleWatchEventsResult> handleAnalysisHandleWatchEvents(AnalysisHandleWatchEventsParams parameters) {
    mirrorLog.info("handleAnalysisHandleWatchEvents ${parameters.toJson()}");
    // for (var e in parameters.events) {}
    parameters.events.forEach((e) {
      if (e.path.startsWith('${ctx.root}/assets')) {
        mirrorLog.info("assets floder change ${e.path}");
        if (e.type == WatchEventType.ADD || e.type == WatchEventType.REMOVE) {
          generateAssetsDartCode(ctx.root, 'add_or_remove');
        }
        // Future.delayed(Duration.zero, () => generateAssetsDartCode(ctx.root));
      }
    });
    return super.handleAnalysisHandleWatchEvents(parameters);
  }
}
